{*_* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

Author:       Franois PIETTE
Description:  Console mode sample program for TIcsSocket.
Creation:     Augustus 14, 2000
Version:      1.00.2 BETA
EMail:        http://www.overbyte.be       francois.piette@overbyte.be
              http://www.rtfm.be/fpiette   francois.piette@rtfm.be
Support:      This code is unsupported.
Legal issues: Copyright (C) 2000-2001 by Franois PIETTE
              Rue de Grady 24, 4053 Embourg, Belgium. Fax: +32-4-365.74.56
              <francois.piette@overbyte.be>

              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. You must register this software by sending a picture postcard
                 to the author. Use a nice stamp and mention your name, street
                 address, EMail address and any comment you like to say.

History:
Feb 26, 2001 V1.00.1 Checked with Kylix V1.0 Build 5.38
Apr 29, 2001 V1.00.2 Use retail Desktop Developer Build 5.62
                     Removed use or errno which doesn't exists anymore

 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
program ThreadTest;
{$APPTYPE CONSOLE}

uses
  SysUtils,
  Classes,
{$IFDEF WIN32}
  Windows,
  Messages,
  WinSock,
{$ELSE}
  Libc,
{$ENDIF}
  IcsSocket,
  IcsSockBuf;

type
    TKbdThreadCommandEvent = procedure (Sender : TObject;
                                           const Cmd : String) of object;
    TKbdThread = class(TThread)
    public
        Buf : String;
        FOnCommand : TKbdThreadCommandEvent;
        procedure Execute; override;
        procedure SendCommand;
        property OnCommand : TKbdThreadCommandEvent read  FOnCommand
                                                       write FOnCommand;
    end;

    TMainObject = class(TComponent)
    public
        IcsSocket1    : TIcsSocket;
        FKbdThread : TKbdThread;
        FTerminated   : Boolean;
        FClients      : TList;
        FCurClient    : Integer;
        MainWnd       : THandle;
        constructor Create(AOwner : TComponent); override;
        destructor  Destroy; override;
        procedure CreateSocket;
        procedure KbdThreadTerminate(sender : TObject);
        procedure KbdThreadCommand(sender : TObject; const Cmd : String);
        procedure IcsSocket1SessionConnected(Sender : TObject; Error  : Word);
        procedure IcsSocket1SessionAvailable(Sender : TObject; Error  : Word);
        procedure IcsSocket1SessionClosed(Sender : TObject; Error  : Word);
        procedure IcsSocket1DataAvailable(Sender : TObject; Error  : Word);
        procedure IcsSocket1DnsLookupDone(Sender : TObject; Error  : Word);
        procedure IcsSocket1ChangeState(Sender : TObject;
                                        OldState : TIcsSocketState;
                                        NewState : TIcsSocketState);
        procedure IcsSocket2DataAvailable(Sender : TObject; Error  : Word);
        procedure IcsSocket2SessionClosed(Sender : TObject; Error  : Word);
        procedure IcsSocket2DataSent(Sender : TObject; Error  : Word);
        procedure Execute;
        procedure Help;
        procedure ApplicationLoop;
        procedure Display(Msg : String);
        procedure SocksConnect5(const Param : String);
        procedure SocksConnect4(const Param : String);
    end;

var
    MainObject : TMainObject;

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
// WriteLn has problems with multithreading. Use low level OS function instead.
procedure ConsoleDisplay(const Msg : String);
{$IFDEF WIN32}
var
    Count : DWORD;
    Buf   : String;
begin
    Buf := Msg + #13#10;
    WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), Buf[1],  Length(Buf), Count, nil);
{$ELSE}
var
    Buf   : String;
begin
    Buf := Msg + #10;
    __write(1, Buf[1], Length(Buf));
{$ENDIF}
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
{$IFNDEF WIN32}
procedure Sleep(MilliSec : LongWord);
begin
    usleep(MilliSec * 1000);
end;
{$ENDIF}


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
function ThreadIdToStr(ID : Cardinal) : String;
begin
{$IFDEF WIN32}
    Result := IntToHex(ID, 8);
{$ELSE}
    Result := IntToStr(ID);
{$ENDIF}
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
{$IFDEF WIN32}
function CheckSynchronize : Boolean;
begin
    result := FALSE;
end;
{$ENDIF}


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TKbdThread.SendCommand;
begin
    try
        FOnCommand(Self, Buf);
    except
        on E:Exception do begin
            ConsoleDisplay('Worker thread exception during command ' +
                           E.ClassName + ': ' + E.Message);
        end;
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TKbdThread.Execute;
begin
    ConsoleDisplay('Hello from KbdThread ! [ID=' +
                   ThreadIdToStr(GetCurrentThreadID) + ']');
    while not Terminated do begin
        Write('Enter a command: ');
        ReadLn(Buf);
        Buf := Trim(Buf);
        if Assigned(FOnCommand) then
            Synchronize(SendCommand);
    end;
    ConsoleDisplay('KbdThread is terminated');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
constructor TMainObject.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    CreateSocket;
    FClients := TList.Create;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
destructor TMainObject.Destroy;
var
    ClientIcsSocket : TIcsSocket;
    I               : Integer;
begin
    if Assigned(IcsSocket1) then begin
        IcsSocket1.Destroy;
        IcsSocket1 := nil;
    end;
    if Assigned(FClients) then begin
        // Destroy all clients recorded in the list
        while FClients.Count > 0 do begin
            ClientIcsSocket := TIcsSocket(FClients.Items[0]);
            ClientIcsSocket.Destroy;
            // Check if client is still on our list
            I := FClients.IndexOf(ClientIcsSocket);
            if I >= 0 then
                FClients.Delete(I);
        end;
        // Destroy client list
        FClients.Destroy;
        FClients := nil;
    end;
    inherited Destroy;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.CreateSocket;
begin
    IcsSocket1                    := TIcsSocket.Create(Self);
    IcsSocket1.Name               := 'IcsSocket1';
    IcsSocket1.OnSessionConnected := IcsSocket1SessionConnected;
    IcsSocket1.OnSessionClosed    := IcsSocket1SessionClosed;
    IcsSocket1.OnDataAvailable    := IcsSocket1DataAvailable;
    IcsSocket1.OnDnsLookupDone    := IcsSocket1DnsLookupDone;
    IcsSocket1.OnSessionAvailable := IcsSocket1SessionAvailable;
    IcsSocket1.OnChangeState      := IcsSocket1ChangeState;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.Display;
begin
    ConsoleDisplay(Msg);
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
{$IFDEF WIN32}
{ This global variable is used to store the windows class characteristic    }
{ and is needed to register the window class used by TWSocket               }
var
    XIcsSocketWindowClass: TWndClass = (
        style         : 0;
        lpfnWndProc   : @DefWindowProc;
        cbClsExtra    : 0;
        cbWndExtra    : 0;
        hInstance     : 0;
        hIcon         : 0;
        hCursor       : 0;
        hbrBackground : 0;
        lpszMenuName  : nil;
        lpszClassName : 'XIcsSocketWindowClass');


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{ Allocate a window handle. This means registering a window class the first }
{ time we are called, and creating a new window each time we are called.    }
function XIcsSocketAllocateHWnd(Obj : TObject): HWND;
var
    TempClass       : TWndClass;
    ClassRegistered : Boolean;
begin
    { Check if the window class is already registered                       }
    XIcsSocketWindowClass.hInstance := HInstance;
    ClassRegistered := GetClassInfo(HInstance,
                                    XIcsSocketWindowClass.lpszClassName,
                                    TempClass);
    if not ClassRegistered then begin
       { Not yet registered, do it right now                                }
       Result := Windows.RegisterClass(XIcsSocketWindowClass);
       if Result = 0 then
           Exit;
    end;

    { Now create a new window                                               }
    Result := CreateWindowEx(WS_EX_TOOLWINDOW,
                             XIcsSocketWindowClass.lpszClassName,
                             '',        { Window name   }
                             WS_POPUP,  { Window Style  }
                             0, 0,      { X, Y          }
                             0, 0,      { Width, Height }
                             0,         { hWndParent    }
                             0,         { hMenu         }
                             HInstance, { hInstance     }
                             nil);      { CreateParam   }
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{ Free the window handle                                                    }
procedure XIcsSocketDeallocateHWnd(Wnd: HWND);
begin
    DestroyWindow(Wnd);
end;
{$ENDIF}


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{$IFDEF WIN32}
procedure TMainObject.ApplicationLoop;
var
    Msg : TMsg;
begin
    MainWnd := XIcsSocketAllocateHWnd(Self);
    try
        while (not FTerminated) and GetMessage(Msg, 0, 0, 0) do begin
            TranslateMessage(Msg);
            DispatchMessage(Msg);
        end;
    finally
        XIcsSocketDeallocateHWnd(MainWnd);
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{$ELSE}
procedure TMainObject.ApplicationLoop;
begin
    while not FTerminated do begin
        while CheckSynchronize do
            ;
        Sleep(0);
    end;
end;
{$ENDIF}


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TMainObject.Execute;
begin
    Display('ThreadTest Start ' + '[ID=' + ThreadIdToStr(GetCurrentThreadID) + ']');
    FKbdThread := TKbdThread.Create(TRUE);
    FKbdThread.OnTerminate := KbdThreadTerminate;
    FKbdThread.OnCommand   := KbdThreadCommand;
    FKbdThread.Resume;
    ApplicationLoop;
    Display('ThreadTest Stop ' + '[ID=' + ThreadIdToStr(GetCurrentThreadID) + ']');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
{ Split a command line into an array of words. Use spaces or tabs as         }
{ command delimiter. Commands words have to be delimited by doubles quotes   }
{ if they include spaces or tabs.                                            }
{ First word in ParamsArray is full parameters, second word is command word  }
{ and subsequent words are individual parameters.                            }
function ParseCommandLine(
    const CmdLine     : String;
    var   ParamsArray : array of string) : Integer;
var
    Index   : Integer;
    I, J    : Integer;
begin
    I                             := 1;
    ParamsArray[Low(ParamsArray)] := '';
    Index                         := Low(ParamsArray) + 1;
    while (Index <= High(ParamsArray)) and
          (I <= Length(CmdLine)) do begin
        { Skip spaces and tabs }
        while (I <= Length(CmdLine)) and (CmdLine[I] in [' ', #9]) do
            Inc(I);
        if I > Length(CmdLine) then
            break;
        if Index = (Low(ParamsArray) + 2) then
            ParamsArray[Low(ParamsArray)] := Copy(CmdLine, I, Length(CmdLine));
        { Check if quoted parameters (can have embeded spaces) }
        if CmdLine[I] = '"' then begin
            Inc(I);
            ParamsArray[Index] := '';
            while I <= Length(CmdLine) do begin
                if CmdLine[I] = '"' then begin
                    if (I >= Length(CmdLine)) or (CmdLine[I + 1] <> '"') then begin
                        Inc(I);
                        break;
                    end;
                    ParamsArray[Index] := ParamsArray[Index] + '"';
                    Inc(I, 2);
                end
                else begin
                    ParamsArray[Index] := ParamsArray[Index] + CmdLine[I];
                    Inc(I);
                end;
            end;
        end
        else begin
            J := I;
            while (I <= Length(CmdLine)) and (not (CmdLine[I] in [' ', #9])) do
                Inc(I);
            if J = I then
                break;
            ParamsArray[Index] := Copy(CmdLine, J, I - J);
        end;
        Inc(Index);
    end;
    Result := Index - Low(ParamsArray);
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TMainObject.SocksConnect5(const Param : String);
const
    AuthMethod : array [Boolean] of TIcsSocksAuthentication =
        (icsSocksNoAuthentication, icsSocksAuthenticateUsercode);
begin
    IcsSocket1.SocksServer         := 'fpiette';
    IcsSocket1.SocksPort           := '1080';
    IcsSocket1.SocksLevel          := '5';
    IcsSocket1.SocksAuthentication := AuthMethod[TRUE];
    IcsSocket1.SocksUsercode       := 'FPIETTE';
    IcsSocket1.SocksPassword       := 'FP';
    IcsSocket1.Proto               := 'tcp';
    IcsSocket1.Port                := 'telnet';
    IcsSocket1.Addr                := 'portable';
    IcsSocket1.Connect;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TMainObject.SocksConnect4(const Param : String);
const
    AuthMethod : array [Boolean] of TIcsSocksAuthentication =
        (icsSocksNoAuthentication, icsSocksAuthenticateUsercode);
begin
    IcsSocket1.SocksServer         := 'fpiette';
    IcsSocket1.SocksPort           := '1080';
    IcsSocket1.SocksLevel          := '4';
    IcsSocket1.SocksAuthentication := icsSocksNoAuthentication;
    IcsSocket1.SocksUsercode       := '';
    IcsSocket1.SocksPassword       := '';
    IcsSocket1.Proto               := 'tcp';
    IcsSocket1.Port                := 'telnet';
    IcsSocket1.Addr                := 'portable';
    IcsSocket1.Connect;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TMainObject.Help;
begin
    Display('Commands are:');
    Display('  Dns hostname|IP');
    Display('  Connect host prt');
    Display('  Send data');
    Display('  SendCli data');
    Display('  Close');
    Display('  CloseDelayed');
    Display('  CloseCli');
    Display('  Listen');
    Display('  SetClient Client#');
    Display('  SocksConnect4');
    Display('  SocksConnect5');
    Display('  Shutdown 0|1|2');
    Display('  ShutdownCli 0|1|2');
    Display('  Destroy');
    Display('  Exit');
    Display('  LineMode on|off');
    Display('  Display on|off');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TMainObject.KbdThreadCommand(sender : TObject; const Cmd : String);
var
    Param       : array [0..10] of String;
    ParamCount  : Integer;
    How         : Integer;
begin
    ParamCount := ParseCommandLine(Cmd, Param);
    if ParamCount <= 0 then
        Exit;
    Param[1] := LowerCase(Param[1]);
    if Param[1] = 'help' then begin
        Help;
    end
    else if Param[1] = 'dns' then begin
        if ParamCount <= 2 then
            Exit;
        IcsSocket1.DnsLookup(Param[2]);
    end
    else if Param[1] = 'connect' then begin
        IcsSocket1.Proto := 'tcp';
        if ParamCount > 2 then
            IcsSocket1.Addr  := Param[2]
        else
            IcsSocket1.Addr  := '192.161.65.25';
        if ParamCount > 3 then
            IcsSocket1.Port  := Param[3]
        else
            IcsSocket1.Port  := 'telnet';
        try
            IcsSocket1.Connect;
            Display('Connecting');
        except
            on E:Exception do
                Display('Unable to connect ' + E.ClassName + ': ' + E.Message);
        end;
    end
    else if Param[1] = 'listen' then begin
        IcsSocket1.Proto := 'tcp';
        IcsSocket1.Addr  := '0.0.0.0';
        if ParamCount > 2 then
            IcsSocket1.Port  := Param[2]
        else
            IcsSocket1.Port  := 'telnet';
        try
            IcsSocket1.Listen;
            Display('Listening Addr=' + IcsSocket1.Addr +
                    ' port=' + IcsSocket1.Port);
        except
            on E:Exception do
                Display('Unable to listen ' + E.ClassName + ': ' + E.Message);
        end;
    end
    else if Param[1] = 'setclient' then begin
        if ParamCount > 2 then
            FCurClient := StrToInt(Param[2]);
        Display('Current client is: ' + IntToStr(FCurClient));
    end
    else if Param[1] = 'socksconnect5' then begin
        SocksConnect5(Param[0]);
    end
    else if Param[1] = 'socksconnect4' then begin
        SocksConnect4(Param[0]);
    end
    else if Param[1] = 'close' then begin
        Display('Closing...');
        IcsSocket1.Close;
    end
    else if Param[1] = 'closedelayed' then begin
        Display('CloseDelayed...');
        IcsSocket1.CloseDelayed;
    end
    else if Param[1] = 'closecli' then begin
        if FCurClient < FClients.Count then begin
            Display('Closing client...');
            TIcsSocket(FClients.Items[FCurClient]).Close;
        end
        else
            Display('No client #' + IntToStr(FCurClient));
    end
    else if Param[1] = 'shutdown' then begin
        Display('Shutdown...');
        if ParamCount > 2 then
            How := StrToInt(Param[2])
        else
            How := 2;
        IcsSocket1.Shutdown(How);
    end
    else if Param[1] = 'shutdowncli' then begin
        if FCurClient < FClients.Count then begin
            Display('Shutdown client...');
            if ParamCount > 2 then
                How := StrToInt(Param[2])
            else
                How := 2;
            TIcsSocket(FClients.Items[FCurClient]).Shutdown(How);
        end
        else
            Display('No client #' + IntToStr(FCurClient));
    end
    else if Param[1] = 'send' then begin
        Display('Sending...');
        IcsSocket1.SendStr(Param[0] + #13#10);
    end
    else if Param[1] = 'sendcli' then begin
        if FCurClient < FClients.Count then begin
            Display('Sending to client...');
            TIcsSocket(FClients.Items[FCurClient]).SendStr(Param[0] + #13#10);
        end
        else
            Display('No client #' + IntToStr(FCurClient));
    end
    else if Param[1] = 'destroy' then begin
        IcsSocket1.Destroy;
        Display('IcsSocket destroyed');
        Display('Creating new socket');
        CreateSocket;
        Display('Socket created');
    end
    else if Param[1] = 'exit' then begin
        FKbdThread.Terminate;
    end
    else if Param[1] = 'display' then begin
        if Param[2] = 'off' then
            IcsSocketDisplayFlag := FALSE
        else
            IcsSocketDisplayFlag := TRUE;
    end
    else if Param[1] = 'linemode' then begin
        if Param[2] = 'off' then
            IcsSocket1.LineMode := FALSE
        else
            IcsSocket1.LineMode := TRUE;
    end
    else begin
        Display('Unknown command ''' + Cmd + '''');
        Help;
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1SessionConnected(
    Sender : TObject;
    Error  : Word);
begin
    if Error <> 0 then
        Display('Unable to connect. Error #' + IntToStr(Error))
    else begin
        Display('Connected !');
        // Display remote infos
        Display('PeerAddr = ' + IcsSocket1.PeerAddr);
        Display('PeerPort = ' + IcsSocket1.PeerPort);
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1SessionAvailable(
    Sender : TObject;
    Error  : Word);
var
    ASocket    : TSocket;
    IcsSocket2 : TIcsSocket;
begin
    if Error <> 0 then
        Display('Unable to listen. Error #' + IntToStr(Error))
    else begin
        Display('Session available !');
        ASocket := IcsSocket1.Accept;
        Display('Accepted socket =' + IntToStr(ASocket));
        // Create a new client IcsSocket
        IcsSocket2 := TIcsSocket.Create(nil);
        IcsSocket2.Name := 'IcsSocketClient' + IntToStr(FClients.Count + 1);
        // add new client to client list
        FClients.Add(IcsSocket2);
        // Initialize new client events handler
        IcsSocket2.OnDataAvailable := IcsSocket2DataAvailable;
        IcsSocket2.OnSessionClosed := IcsSocket2SessionClosed;
        IcsSocket2.OnDataSent      := IcsSocket2DataSent;
        IcsSocket2.OnChangeState   := IcsSocket1ChangeState;
        // Initialize LineMode
        IcsSocket2.LineMode := IcsSocket1.LineMode;
        IcsSocket2.LineEnd  := IcsSocket2.LineEnd;
        IcsSocket2.LineEdit := IcsSocket2.LineEdit;
        // Activate new client with accepted socket
        IcsSocket2.Dup(ASocket);
Display('Dup OK');
        // Display remote infos
        Display('PeerAddr = ' + IcsSocket2.PeerAddr);
        Display('PeerPort = ' + IcsSocket2.PeerPort);
Display('Dup Finished');
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1DnsLookupDone(
    Sender : TObject;
    Error  : Word);
begin
    if Error <> 0 then
        Display('DNS lookup failed. Error #' + IntToStr(Error))
    else
        Display('DNS lookup done. Result = ''' + IcsSocket1.DnsResult + '''');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1ChangeState(
    Sender : TObject;
    OldState : TIcsSocketState;
    NewState : TIcsSocketState);
var
    IcsSocket : TIcsSocket;
begin
    IcsSocket := Sender as TIcsSocket;
    Display(IcsSocket.Name + ': State Change ' + IntToStr(Ord(OldState)) +
            ' -> ' + IntToStr(Ord(NewState)));
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1SessionClosed(
    Sender : TObject;
    Error  : Word);
begin
    if Error <> 0 then
        Display('Session closed. Error #' + IntToStr(Error))
    else
        Display('Closed.');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket1DataAvailable(
    Sender : TObject;
    Error  : Word);
var
    Buf   : array [0..1023] of char;
    Count : Integer;
    Len   : LongInt;
begin
    Len := IcsSocket1.RcvdCount;
    Display('RcvdCount = ' + IntToStr(Len));
    Count := IcsSocket1.Receive(@Buf, SizeOf(Buf));
    if Count > 0 then begin
        Buf[Count] := #0;
        Display('Received ' + IntToStr(Count) + ' bytes: ''' + Buf + '''');
    end;
    if IcsSocket1.LineMode and (Buf = 'off'#13#10) then
        IcsSocket1.LineMode := FALSE;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket2DataAvailable(
    Sender : TObject;
    Error  : Word);
var
    Buf       : array [0..2047] of char;
    Count     : Integer;
    IcsSocket : TIcsSocket;
begin
    IcsSocket := Sender as TIcsSocket;
    Count     := IcsSocket.Receive(@Buf, SizeOf(Buf) - 1);
    if Count > 0 then begin
        Buf[Count] := #0;
        Display('Received from client ' + IcsSocket.Name + ' ' +
                IntToStr(Count) + ' bytes: ''' + Buf + '''');
    end
    else if Count <= 0 then begin
        Display('No data received from client ' + IcsSocket.Name +
                '. Error #' + IntToStr(Error));
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket2DataSent(
    Sender : TObject;
    Error  : Word);
var
    IcsSocket : TIcsSocket;
//const
//    TestCnt : Integer = 5;
begin
    IcsSocket := Sender as TIcsSocket;
    if Error <> 0 then
        Display('Client ' + IcsSocket.Name +
                ' DataSent. Error #' + IntToStr(Error))
    else
        Display('Client ' + IcsSocket.Name + ' DataSent.');
//    if TestCnt > 0 then begin
//        Dec(TestCnt);
//        IcsSocket.SendStr('TestCnt = ' + IntToStr(TestCnt) + #13#10);
//    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.IcsSocket2SessionClosed(
    Sender : TObject;
    Error  : Word);
var
    IcsSocket : TIcsSocket;
    I         : Integer;
begin
    IcsSocket := Sender as TIcsSocket;
    if Error <> 0 then
        Display('Client ' + IcsSocket.Name +
                ' closed. Error #' + IntToStr(Error))
    else
        Display('Client ' + IcsSocket.Name + ' Closed.');

    // Remove client from client list
    I := FClients.IndexOf(IcsSocket);
    if I >= 0 then
        FClients.Delete(I);
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
procedure TMainObject.KbdThreadTerminate(sender : TObject);
begin
    FTerminated := TRUE;
{$IFDEF WIN32}
    PostMessage(MainWnd, WM_QUIT, 0, 0);
{$ENDIF}
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *  * * * * * * * *}
begin
    IcsDisplayProc := ConsoleDisplay;
    MainObject := TMainObject.Create(nil);
    try
        MainObject.Execute;
    finally
        MainObject.Destroy;
    end;
end.

